@@ -24,11 +24,16 @@ output = weather_agent.run("Hello. What is the weather in Tokyo?")
 
 from __future__ import annotations
 
-
 import datetime
 import json
-from collections.abc import AsyncGenerator, Callable, MutableMapping, Sequence
-from contextlib import asynccontextmanager
+from collections.abc import (
+    AsyncGenerator,
+    Callable,
+    Generator,
+    MutableMapping,
+    Sequence,
+)
+from contextlib import asynccontextmanager, contextmanager
 from typing import TYPE_CHECKING, Any, Literal, cast
 
 from mcp.types import Tool as MCPTool
@@ -38,19 +43,19 @@ from rsb.models.config_dict import ConfigDict
 from rsb.models.field import Field
 from rsb.models.mimetype import MimeType
 
+from agentle.agents.a2a.models.agent_skill import AgentSkill
+from agentle.agents.a2a.models.agent_usage_statistics import AgentUsageStatistics
+from agentle.agents.a2a.models.artifact import Artifact
+from agentle.agents.a2a.models.authentication import Authentication
+from agentle.agents.a2a.models.capabilities import Capabilities
+from agentle.agents.a2a.models.run_state import RunState
+from agentle.agents.a2a.tasks.task_state import TaskState
 from agentle.agents.agent_config import AgentConfig
 from agentle.agents.agent_run_output import AgentRunOutput
 from agentle.agents.context import Context
 from agentle.agents.errors.max_tool_calls_exceeded_error import (
     MaxToolCallsExceededError,
 )
-from agentle.agents.models.agent_skill import AgentSkill
-from agentle.agents.models.agent_usage_statistics import AgentUsageStatistics
-from agentle.agents.models.artifact import Artifact
-from agentle.agents.models.authentication import Authentication
-from agentle.agents.models.capabilities import Capabilities
-from agentle.agents.models.run_state import RunState
-from agentle.agents.tasks.task_state import TaskState
 from agentle.generations.collections.message_sequence import MessageSequence
 from agentle.generations.models.generation.generation import Generation
 from agentle.generations.models.message_parts.file import FilePart
@@ -66,6 +71,7 @@ from agentle.generations.providers.base.generation_provider import (
     GenerationProvider,
 )
 from agentle.generations.tools.tool import Tool
+from agentle.generations.tracing.langfuse import LangfuseObservabilityClient
 from agentle.mcp.servers.mcp_server_protocol import MCPServerProtocol
 
 if TYPE_CHECKING:
@@ -129,6 +135,7 @@ class Agent[T_Schema = WithoutStructuredOutput](BaseModel):
         url: URL where the agent is hosted.
         generation_provider: Generation provider used by the agent.
         version: Version of the agent.
+        endpoint: Endpoint of the agent.
         documentationUrl: URL to agent documentation.
         capabilities: Optional capabilities supported by the agent.
         authentication: Authentication requirements for the agent.
@@ -183,16 +190,30 @@ class Agent[T_Schema = WithoutStructuredOutput](BaseModel):
     A URL to the address the agent is hosted at.
     """
 
-    generation_provider: GenerationProvider
+    generation_provider: GenerationProvider | type[GenerationProvider]
     """
     The service provider of the agent
     """
 
-    version: str = Field(default="0.0.1")
+    version: str = Field(
+        default="0.0.1",
+        description="The version of the agent - format is up to the provider. (e.g. '1.0.0')",
+        examples=["1.0.0", "1.0.1", "1.1.0"],
+        pattern=r"^\d+\.\d+\.\d+$",
+    )
     """
     The version of the agent - format is up to the provider. (e.g. "1.0.0")
     """
 
+    endpoint: str = Field(
+        default=f"/api/v1/agents/{name}",
+        description="The endpoint of the agent",
+        examples=["/api/v1/agents/weather-agent"],
+    )
+    """
+    The endpoint of the agent
+    """
+
     documentationUrl: str | None = Field(default=None)
     """
     A URL to documentation for the agent.
@@ -233,7 +254,7 @@ class Agent[T_Schema = WithoutStructuredOutput](BaseModel):
     """
 
     # Library-specific fields
-    model: str
+    model: str | None = Field(default=None)
     """
     The model to use for the agent's service provider.
     """
@@ -287,8 +308,35 @@ class Agent[T_Schema = WithoutStructuredOutput](BaseModel):
         """
         return len(self.tools) > 0
 
+    @contextmanager
+    def with_mcp_servers(self) -> Generator[None, None, None]:
+        """
+        Context manager to connect and clean up MCP servers.
+
+        This context manager ensures that all MCP servers are connected before the
+        code block is executed and cleaned up after completion, even in case of exceptions.
+
+        Yields:
+            None: Does not return a value, just manages the context.
+
+        Example:
+            ```python
+            async with agent.with_mcp_servers():
+                # Operations that require connection to MCP servers
+                result = await agent.run_async("Query to server")
+            # Servers are automatically disconnected here
+            ```
+        """
+        for server in self.mcp_servers:
+            run_sync(server.connect)
+        try:
+            yield
+        finally:
+            for server in self.mcp_servers:
+                run_sync(server.cleanup)
+
     @asynccontextmanager
-    async def with_mcp_servers(self) -> AsyncGenerator[None, None]:
+    async def async_with_mcp_servers(self) -> AsyncGenerator[None, None]:
         """
         Asynchronous context manager to connect and clean up MCP servers.
 
@@ -389,6 +437,12 @@ class Agent[T_Schema = WithoutStructuredOutput](BaseModel):
             input, instructions=self._convert_instructions_to_str(self.instructions)
         )
 
+        generation_provider: GenerationProvider = (
+            self.generation_provider
+            if isinstance(self.generation_provider, GenerationProvider)
+            else self.generation_provider()
+        )
+
         mcp_tools: list[MCPTool] = []
         for server in self.mcp_servers:
             tools = await server.list_tools()
@@ -398,7 +452,7 @@ class Agent[T_Schema = WithoutStructuredOutput](BaseModel):
         if not agent_has_tools:
             generation: Generation[
                 T_Schema
-            ] = await self.generation_provider.create_generation_async(
+            ] = await generation_provider.create_generation_async(
                 model=self.model,
                 messages=context.messages,
                 response_schema=self.response_schema,
@@ -406,7 +460,7 @@ class Agent[T_Schema = WithoutStructuredOutput](BaseModel):
             )
 
             return AgentRunOutput[T_Schema](
-                model_name=self.model,
+                model_name=generation.model,
                 artifacts=[
                     Artifact(
                         name="Artifact",
@@ -475,7 +529,7 @@ class Agent[T_Schema = WithoutStructuredOutput](BaseModel):
 
             no_more_tools = len(filtered_tools) == 0
             if no_more_tools:
-                generation = await self.generation_provider.create_generation_async(
+                generation = await generation_provider.create_generation_async(
                     model=self.model,
                     messages=MessageSequence(context.messages)
                     .append_before_last_message(called_tools_prompt)
@@ -491,20 +545,18 @@ class Agent[T_Schema = WithoutStructuredOutput](BaseModel):
                     context=context,
                     generation=generation,
                 )
-            tool_call_generation = (
-                await self.generation_provider.create_generation_async(
-                    model=self.model,
-                    messages=MessageSequence(context.messages)
-                    .append_before_last_message(called_tools_prompt)
-                    .elements,
-                    generation_config=self.config.generationConfig,
-                    tools=filtered_tools,
-                )
+            tool_call_generation = await generation_provider.create_generation_async(
+                model=self.model,
+                messages=MessageSequence(context.messages)
+                .append_before_last_message(called_tools_prompt)
+                .elements,
+                generation_config=self.config.generationConfig,
+                tools=filtered_tools,
             )
 
             agent_didnt_call_any_tool = tool_call_generation.tool_calls_amount() == 0
             if agent_didnt_call_any_tool:
-                generation = await self.generation_provider.create_generation_async(
+                generation = await generation_provider.create_generation_async(
                     model=self.model,
                     messages=context.messages,
                     response_schema=self.response_schema,
@@ -678,7 +730,7 @@ class Agent[T_Schema = WithoutStructuredOutput](BaseModel):
         parsed = generation.parsed
 
         return AgentRunOutput(
-            model_name=self.model,
+            model_name=self.model or generation.model,
             artifacts=artifacts
             or [
                 Artifact(
@@ -968,10 +1020,22 @@ class Agent[T_Schema = WithoutStructuredOutput](BaseModel):
 
 
 if __name__ == "__main__":
+    import pydantic
+
     from agentle.generations.providers.google.google_generation_provider import (
         GoogleGenerationProvider,
     )
-    import pydantic
+    from agentle.mcp.servers.http_mcp_server import HTTPMCPServer
+    from dotenv import load_dotenv
+
+    load_dotenv(override=True)
+
+    class FakeHTTPMCPServer(HTTPMCPServer):
+        async def connect(self) -> None:
+            print("Connected to fake MCP server")
+
+        async def cleanup(self) -> None:
+            print("Disconnected from fake MCP server")
 
     class Weather(pydantic.BaseModel):
         location: str
@@ -981,15 +1045,17 @@ if __name__ == "__main__":
         return Weather(location=location, weather="sunny")
 
     weather_agent = Agent(
-        generation_provider=GoogleGenerationProvider(),
-        model="gemini-2.0-flash",
+        generation_provider=GoogleGenerationProvider(
+            tracing_client=LangfuseObservabilityClient()
+        ),
         instructions="You are a weather agent that can answer questions about the weather.",
         tools=[get_weather],
         response_schema=Weather,
     )
 
-    output = weather_agent.run(
-        "Hello. What is the weather in Tokyo? what do you think about tokio?"
-    )
+    with weather_agent.with_mcp_servers():
+        output = weather_agent.run(
+            "Hello. What is the weather in Tokyo? what do you think about tokio?"
+        )
 
     print(output)
